(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     48604,       1363]
NotebookOptionsPosition[     44921,       1243]
NotebookOutlinePosition[     45344,       1259]
CellTagsIndexPosition[     45301,       1256]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[{
 StyleBox["Methoden der Bioinformatik",
  FontSize->24,
  FontWeight->"Bold"],
 StyleBox[" \n\n",
  FontSize->12],
 "Eine Einf\[UDoubleDot]hrung zur Anwendung in Biologie und Medizin\n\n2. \
Auflage",
 StyleBox["\n",
  FontSize->18,
  FontWeight->"Bold"],
 StyleBox["\n",
  FontSize->12],
 "Marc-Thorsten H\[UDoubleDot]tt",
 StyleBox[" ",
  FontSize->12,
  FontWeight->"Bold"],
 "und",
 StyleBox[" ",
  FontSize->12,
  FontWeight->"Bold"],
 "Manuel Dehnert\n",
 StyleBox["\n",
  FontSize->12],
 "Springer Spektrum",
 StyleBox[" ",
  FontSize->12,
  FontWeight->"Bold"],
 "2006",
 StyleBox[", ",
  FontSize->12,
  FontWeight->"Bold"],
 "2016, ISBN 978-3-662-46149-5"
}], "Subsubtitle",
 CellChangeTimes->{{3.669868691792925*^9, 3.669868786167323*^9}, {
  3.6698688284727426`*^9, 3.669868879042635*^9}, {3.6698691142260866`*^9, 
  3.6698691148331213`*^9}}],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "-Exkurs: ",
 "Viterbi-Algorithmus"
}], "Subtitle",
 CellChangeTimes->{3.670321495640956*^9, 3.6703245549045925`*^9},
 Background->RGBColor[
  0.8705882352941177, 0.00392156862745098, 0.2196078431372549]],

Cell[TextData[{
 "In dem vorangegangenen Abschnitt wurde der Viterbi-Algorithmus eingef\
\[UDoubleDot]hrt und am Beispiel eines Hidden-Markov-Modells f\[UDoubleDot]r \
ein gelegentlich unehrliches Kasino erl\[ADoubleDot]utert. Dieses Modell soll \
uns nun auch als Beispiel f\[UDoubleDot]r die Implementierung des \
Viterbi-Algorithmus in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " dienen. Im ersten Schritt wird das Modell durch den Zustandsraum des HMM, \
den Zustandsraum der beobachteten Sequenz, die \
\[CapitalUDoubleDot]bergangswahrscheinlichkeiten und die \
Emissionswahrscheinlichkeiten spezifiziert. \n\nDer Zustandsraum des HMM wird \
festgelegt als"
}], "Text",
 CellChangeTimes->{{3.6746248369318867`*^9, 3.6746248670929317`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"state", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<0\>\"", ",", "\"\<F\>\"", ",", "\"\<U\>\""}], "}"}]}], 
  ";"}]], "Input"],

Cell["\<\
wobei \[OpenCurlyQuote]0\[CloseCurlyQuote] f\[UDoubleDot]r den Anfangszustand \
steht, \[OpenCurlyQuote]F\[CloseCurlyQuote] f\[UDoubleDot]r den fairen und \
\[OpenCurlyQuote]U\[CloseCurlyQuote] f\[UDoubleDot]r den unfairen W\
\[UDoubleDot]rfel. Der Zustandsraum der Sequenz ist der eines \
gew\[ODoubleDot]hnlichen W\[UDoubleDot]rfels.\
\>", "Text",
 CellChangeTimes->{{3.674624877131947*^9, 3.674624890441966*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"observation", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}]}], 
  ";"}]], "Input"],

Cell["\<\
Die \[CapitalUDoubleDot]bergangswahrscheinlichkeiten zwischen den drei Zust\
\[ADoubleDot]nden \[OpenCurlyQuote]0\[CloseCurlyQuote], \[OpenCurlyQuote]F\
\[CloseCurlyQuote] und \[OpenCurlyQuote]U\[CloseCurlyQuote] werden in Form \
einer Funktion definiert. Die Argumente sind die Zust\[ADoubleDot]nde und der \
ausgegebene Funktionswert ist die zugeh\[ODoubleDot]rige \
\[CapitalUDoubleDot]bergangswahrscheinlichkeit.\
\>", "Text",
 CellChangeTimes->{{3.674624914642004*^9, 3.6746249251520205`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"transition", "[", 
    RowBox[{"z1_", ",", "z2_"}], "]"}], ":=", 
   RowBox[{"Switch", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"z1", ",", "z2"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<F\>\"", ",", "\"\<F\>\""}], "}"}], ",", "0.95", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<F\>\"", ",", "\"\<U\>\""}], "}"}], ",", "0.05", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<U\>\"", ",", "\"\<U\>\""}], "}"}], ",", "0.90", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<U\>\"", ",", "\"\<F\>\""}], "}"}], ",", "0.10", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<0\>\"", ",", "\"\<U\>\""}], "}"}], ",", "0.5", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<0\>\"", ",", "\"\<F\>\""}], "}"}], ",", "0.5", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<0\>\"", ",", "\"\<0\>\""}], "}"}], ",", "0.0", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<U\>\"", ",", "\"\<0\>\""}], "}"}], ",", "0.0", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<F\>\"", ",", "\"\<0\>\""}], "}"}], ",", "0.0"}], "]"}]}], 
  ";"}]], "Input"],

Cell["\<\
In gleicher Weise werden auch die Emissionswahrscheinlichkeiten festgelegt.\
\>", "Text",
 CellChangeTimes->{{3.674624941121045*^9, 3.6746249559590673`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"emission", "[", 
    RowBox[{"z1_", ",", "z2_"}], "]"}], ":=", 
   RowBox[{"Switch", "[", 
    RowBox[{"z1", ",", "\[IndentingNewLine]", "\"\<F\>\"", ",", 
     RowBox[{"N", "[", 
      RowBox[{"1", "/", "6"}], "]"}], ",", "\[IndentingNewLine]", "\"\<U\>\"",
      ",", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"z2", "\[Equal]", "6"}], ",", "0.5", ",", "0.1"}], "]"}], ",", 
     "\[IndentingNewLine]", "\"\<0\>\"", ",", "0"}], "]"}]}], ";"}]], "Input"],

Cell[TextData[{
 "Dabei ist ",
 StyleBox["z1",
  FontWeight->"Bold"],
 " erneut ein Symbol aus dem HMM-Zustandsraum state, die \
Gr\[ODoubleDot]\[SZ]e ",
 StyleBox["z2",
  FontWeight->"Bold"],
 " bezeichnet jetzt jedoch ein Symbol aus dem Zustandsraum ",
 StyleBox["observation",
  FontWeight->"Bold"],
 " der beobachteten Sequenz. Wir wollen nun die von uns definierten Parameter \
des Modells stichprobenartig \[UDoubleDot]berpr\[UDoubleDot]fen und dabei die \
Funktionsweise der Implementierung verdeutlichen. Die Wahrscheinlichkeit, mit \
einem fairen W\[UDoubleDot]rfel eine Drei zu w\[UDoubleDot]rfeln, ist demnach \
gegeben durch"
}], "Text",
 CellChangeTimes->{{3.6746250134221525`*^9, 3.6746250328081827`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"emission", "[", 
  RowBox[{"\"\<F\>\"", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["0.16666666666666666`"], "Output",
 CellChangeTimes->{3.6775712285118856`*^9}]
}, Open  ]],

Cell["\<\
Die Wahrscheinlichkeit, von einem fairen W\[UDoubleDot]rfel im \
n\[ADoubleDot]chsten Schritt zu einem unfairen W\[UDoubleDot]rfel \
\[UDoubleDot]berzugehen, ist\
\>", "Text",
 CellChangeTimes->{{3.6746250749052505`*^9, 3.674625087550269*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"transition", "[", 
  RowBox[{"\"\<F\>\"", ",", "\"\<U\>\""}], "]"}]], "Input"],

Cell[BoxData["0.05`"], "Output",
 CellChangeTimes->{3.677571228558686*^9}]
}, Open  ]],

Cell[TextData[{
 "Als N\[ADoubleDot]chstes ben\[ODoubleDot]tigen wir eine Beispielsequenz, \
auf die der Viterbi-Algorithmus angewendet werden kann. F\[UDoubleDot]r \
dieses einf\[UDoubleDot]hrende Beispiel erzeugen wir drei Teilsequenzen mit \
unterschiedlichen Eigenschaften. Die erste und die dritte Teilsequenz sind \
Realisierungen eines fairen W\[UDoubleDot]rfels, d.h. ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", 
    StyleBox["Fi",
     FontSlant->"Italic"]], TraditionalForm]]],
 " = 1/6 \[ForAll] i \[Element] \[CapitalSigma]. Die zweite Teilsequenz ist \
eine Realisierung eines unfairen W\[UDoubleDot]rfels mit den \
Wahrscheinlichkeiten ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "U6"], TraditionalForm]]],
 " = 1/2 und ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", 
    StyleBox["Ui",
     FontSlant->"Italic"]], TraditionalForm]]],
 " = 1/ 10, i = 1, . . . , 5."
}], "Text",
 CellChangeTimes->{{3.67462511927532*^9, 3.6746251241293273`*^9}, {
  3.674625155406374*^9, 3.6746252230654745`*^9}}],

Cell[BoxData[
 RowBox[{"SeedRandom", "[", "2", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"seq0a", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"RandomInteger", "[", 
     RowBox[{"{", 
      RowBox[{"1", ",", "6"}], "}"}], "]"}], ",", 
    RowBox[{"{", "50", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.622880508576276*^9, 3.622880512809978*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "6", ",", "2", ",", "3", ",", "3", ",", "6", ",", "3", ",", "2", ",", "6", 
   ",", "6", ",", "1", ",", "1", ",", "5", ",", "4", ",", "5", ",", "1", ",", 
   "2", ",", "2", ",", "6", ",", "2", ",", "6", ",", "5", ",", "5", ",", "1", 
   ",", "1", ",", "5", ",", "5", ",", "2", ",", "3", ",", "4", ",", "4", ",", 
   "1", ",", "2", ",", "1", ",", "5", ",", "3", ",", "3", ",", "2", ",", "6", 
   ",", "5", ",", "4", ",", "1", ",", "6", ",", "6", ",", "3", ",", "3", ",", 
   "2", ",", "3", ",", "6", ",", "3", ",", "3"}], "}"}]], "Output",
 CellChangeTimes->{3.6775712285898857`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"seq0b", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"RandomReal", "[", "]"}], "<", "0.5"}], ",", "6", ",", 
      RowBox[{"RandomInteger", "[", 
       RowBox[{"{", 
        RowBox[{"1", ",", "5"}], "}"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", "20", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.62288051619455*^9, 3.6228805295903683`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "5", ",", "2", ",", "6", ",", "5", ",", "6", ",", "6", ",", "6", ",", "6", 
   ",", "6", ",", "5", ",", "2", ",", "1", ",", "6", ",", "6", ",", "3", ",", 
   "5", ",", "6", ",", "2", ",", "6", ",", "6"}], "}"}]], "Output",
 CellChangeTimes->{3.677571228621086*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"seq0c", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"RandomInteger", "[", 
     RowBox[{"{", 
      RowBox[{"1", ",", "6"}], "}"}], "]"}], ",", 
    RowBox[{"{", "50", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.622880536233346*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "6", ",", "5", ",", "5", ",", "6", ",", "3", ",", "5", ",", "4", 
   ",", "6", ",", "3", ",", "1", ",", "4", ",", "3", ",", "3", ",", "4", ",", 
   "3", ",", "4", ",", "3", ",", "4", ",", "6", ",", "2", ",", "5", ",", "3", 
   ",", "3", ",", "3", ",", "4", ",", "2", ",", "6", ",", "5", ",", "1", ",", 
   "4", ",", "3", ",", "1", ",", "1", ",", "2", ",", "1", ",", "5", ",", "3", 
   ",", "3", ",", "1", ",", "5", ",", "1", ",", "3", ",", "1", ",", "3", ",", 
   "6", ",", "3", ",", "2", ",", "3", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.6775712286366863`*^9}]
}, Open  ]],

Cell["\<\
Durch das einfache Aneinanderh\[ADoubleDot]ngen der Teilsequenzen erh\
\[ADoubleDot]lt man eine Sequenz, die zwei unterschiedliche verborgene Zust\
\[ADoubleDot]nde aufweist.\
\>", "Text",
 CellChangeTimes->{{3.67462525392352*^9, 3.6746252619325323`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"seq", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"seq0a", ",", "seq0b", ",", "seq0c"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"L", "=", 
  RowBox[{"Length", "[", "seq", "]"}]}]], "Input"],

Cell[BoxData["120"], "Output",
 CellChangeTimes->{3.677571228667886*^9}]
}, Open  ]],

Cell[TextData[{
 "Die interne Zustandssequenz, die wir in diesem konstruierten Beispiel genau \
kennen, legen wir an dieser Stelle in der Variablen ",
 StyleBox["label",
  FontWeight->"Bold"],
 " ab und werden sie am Ende des ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "-Exkurses mit der vom Viterbi-Algorithmus bestimmten Sequenz der internen \
Zust\[ADoubleDot]nde vergleichen."
}], "Text",
 CellChangeTimes->{{3.6746253095216026`*^9, 3.674625324796625*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"label", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{"\"\<F\>\"", ",", 
        RowBox[{"{", "50", "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<U\>\"", ",", 
        RowBox[{"{", "20", "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<F\>\"", ",", 
        RowBox[{"{", "50", "}"}]}], "]"}]}], "}"}], "]"}]}], ";"}]], "Input"],

Cell["Der Initialisierung der Viterbi-Variablen", "Text",
 CellChangeTimes->{{3.6746253487146597`*^9, 3.674625360557678*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vit", "[", 
   RowBox[{"z1_", ",", "0", ",", "x_"}], "]"}], ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"z1", "==", "\"\<0\>\""}], ",", "1", ",", "0"}], "]"}]}]], "Input"],

Cell["\<\
folgt die Rekursion. Dabei wird die Viterbi-Variable f\[UDoubleDot]r jeden \
Zustand entlang der Sequenz bestimmt.\
\>", "Text",
 CellChangeTimes->{{3.674625363814684*^9, 3.6746253849897156`*^9}}],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"tmpList", "=", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"vit", "[", 
              RowBox[{
               RowBox[{"state", "[", 
                RowBox[{"[", "k", "]"}], "]"}], ",", 
               RowBox[{"i", "-", "1"}], ",", "seq"}], "]"}], "*", 
             RowBox[{"transition", "[", 
              RowBox[{
               RowBox[{"state", "[", 
                RowBox[{"[", "k", "]"}], "]"}], ",", 
               RowBox[{"state", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}], "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"k", ",", "1", ",", 
              RowBox[{"Length", "[", "state", "]"}]}], "}"}]}], "]"}]}], 
         ";"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"vit", "[", 
           RowBox[{
            RowBox[{"state", "[", 
             RowBox[{"[", "j", "]"}], "]"}], ",", "i", ",", "seq"}], "]"}], 
          "=", 
          RowBox[{
           RowBox[{"emission", "[", 
            RowBox[{
             RowBox[{"state", "[", 
              RowBox[{"[", "j", "]"}], "]"}], ",", 
             RowBox[{"seq", "[", 
              RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "*", 
           RowBox[{"Max", "[", "tmpList", "]"}]}]}], ";"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"pointer", "[", 
          RowBox[{"j", ",", 
           RowBox[{"i", "-", "1"}]}], "]"}], "=", 
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{"Position", "[", 
            RowBox[{"tmpList", ",", 
             RowBox[{"Max", "[", "tmpList", "]"}]}], "]"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}]}], "}"}], ";"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", "state", "]"}]}], "}"}]}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "L"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.574150152375*^9, 3.574150152828125*^9}}],

Cell[TextData[{
 "Man erkennt im Inneren der beiden ",
 StyleBox["Do",
  FontWeight->"Bold"],
 "-Schleifen (eine f\[UDoubleDot]r alle HMM-Zust\[ADoubleDot]nde, eine f\
\[UDoubleDot]r den Index entlang der Sequenz) deutlich die Umsetzung von Gl. \
(2.86). Dabei wird die Liste f\[UDoubleDot]r die Maximumsabfrage zuerst in \
der Variablen ",
 StyleBox["tmpList",
  FontWeight->"Bold"],
 " abgelegt. Die Position des Maximums dient als Codierung f\[UDoubleDot]r \
die Zust\[ADoubleDot]nde \[OpenCurlyQuote]0\[CloseCurlyQuote], \
\[OpenCurlyQuote]F\[CloseCurlyQuote], \[OpenCurlyQuote]U\[CloseCurlyQuote] \
und wird in der Variablen ",
 StyleBox["pointer",
  FontWeight->"Bold"],
 " gespeichert. Der Zustand \[OpenCurlyQuote]0\[CloseCurlyQuote] wird durch 1 \
codiert, \[OpenCurlyQuote]F\[CloseCurlyQuote] durch 2 und \[OpenCurlyQuote]U\
\[CloseCurlyQuote] durch 3. Diese Struktur sollte aufgrund unserer \
vorangegangenen Diskussionen sehr klar sein. Die drei Befehle in der \
geschweiften Klammer im Innern der ",
 StyleBox["Do",
  FontWeight->"Bold"],
 "-Schleife sind: (1) die Liste f\[UDoubleDot]r die Maximumsabfrage, (2) die \
Umsetzung von Gl. (2.86) und (3) die Zeigerzuweisung. Mit dem Befehl\n\n",
 StyleBox["Table[vit[state[[k]], i, seq], {k, 1, Length[state]}]",
  FontWeight->"Bold"],
 "\n\nl\[ADoubleDot]sst sich die Viterbi-Variable der Zust\[ADoubleDot]nde 1 \
(\[OpenCurlyQuote]0\[CloseCurlyQuote]), 2 (\[OpenCurlyQuote]F\
\[CloseCurlyQuote]), und 3 (\[OpenCurlyQuote]U\[CloseCurlyQuote]) f\
\[UDoubleDot]r den Sequenzindex i ausgeben. Einige Beispiele:"
}], "Text",
 CellChangeTimes->{{3.6746255912340198`*^9, 3.67462559849903*^9}, 
   3.6746256571771193`*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"i", "=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"vit", "[", 
    RowBox[{
     RowBox[{"state", "[", 
      RowBox[{"[", "k", "]"}], "]"}], ",", "i", ",", "seq"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "1", ",", 
     RowBox[{"Length", "[", "state", "]"}]}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.6775712287926865`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"i", "=", "1"}], ";"}], "\n", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"vit", "[", 
    RowBox[{
     RowBox[{"state", "[", 
      RowBox[{"[", "k", "]"}], "]"}], ",", "i", ",", "seq"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "1", ",", 
     RowBox[{"Length", "[", "state", "]"}]}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.`", ",", "0.08333333333333333`", ",", "0.25`"}], "}"}]], "Output",
 CellChangeTimes->{3.6775712288238864`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"i", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"vit", "[", 
    RowBox[{
     RowBox[{"state", "[", 
      RowBox[{"[", "k", "]"}], "]"}], ",", "i", ",", "seq"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "1", ",", 
     RowBox[{"Length", "[", "state", "]"}]}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.`", ",", "0.013194444444444443`", ",", "0.022500000000000003`"}],
   "}"}]], "Output",
 CellChangeTimes->{3.677571228839486*^9}]
}, Open  ]],

Cell[TextData[{
 "Wir gelangen nun zur Terminierung. F\[UDoubleDot]r den Index ",
 StyleBox["i = L",
  FontSlant->"Italic"],
 " erh\[ADoubleDot]lt man die Viterbi-Variablen f\[UDoubleDot]r das \
Sequenzende und den letzten Wert der Zeigervariablen."
}], "Text",
 CellChangeTimes->{{3.674625704749191*^9, 3.67462573203623*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tmpList", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"vit", "[", 
     RowBox[{
      RowBox[{"state", "[", 
       RowBox[{"[", "k", "]"}], "]"}], ",", "L", ",", "seq"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "1", ",", 
      RowBox[{"Length", "[", "state", "]"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "8.278964493166057`*^-96", ",", "2.6144098399471766`*^-97"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6775712288706865`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pointerFinal", "=", 
  RowBox[{
   RowBox[{"Flatten", "[", 
    RowBox[{"Position", "[", 
     RowBox[{"tmpList", ",", 
      RowBox[{"Max", "[", "tmpList", "]"}]}], "]"}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input"],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.6775712289018865`*^9}]
}, Open  ]],

Cell[TextData[{
 "Die ",
 StyleBox["Do",
  FontWeight->"Bold"],
 "-Schleife ist die direkte Umsetzung von Gl. (2.90). Von dem Zustand aus, \
der zum Maximum der Viterbi-Variablen f\[UDoubleDot]hrt, wird nun mithilfe \
des Traceback-Verfahrens der wahrscheinlichste Pfad ermittelt und ausgegeben."
}], "Text",
 CellChangeTimes->{{3.674625769278287*^9, 3.674625779026301*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"pathX", "[", "L", "]"}], "=", "pointerFinal"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"pathX", "[", 
     RowBox[{"i", "-", "1"}], "]"}], "=", 
    RowBox[{"pointer", "[", 
     RowBox[{
      RowBox[{"pathX", "[", "i", "]"}], ",", 
      RowBox[{"i", "-", "1"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "L", ",", "1", ",", 
     RowBox[{"-", "1"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.574150248234375*^9, 3.5741502489375*^9}, {
  3.57415069971875*^9, 3.574150709390625*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"path", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"pathX", "[", "i", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "L"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5741507134375*^9, 3.57415071665625*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "3", 
   ",", "3", ",", "3", ",", "3", ",", "3", ",", "3", ",", "3", ",", "3", ",", 
   "3", ",", "3", ",", "3", ",", "3", ",", "3", ",", "3", ",", "3", ",", "3", 
   ",", "3", ",", "3", ",", "3", ",", "3", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.677571228964287*^9}]
}, Open  ]],

Cell["\<\
Die Visualisierung des Ergebnisses erfolgt durch eine etwas aufwendige \
Funktion, an der man gut einige fortgeschrittene Formatierungsfragen vorf\
\[UDoubleDot]hren kann:\
\>", "Text",
 CellChangeTimes->{{3.6746258646084256`*^9, 3.6746258777924447`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeViterbiOutput", "[", 
   RowBox[{"sequenz_", ",", "rowLength_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"tmp1", ",", "tmp2", ",", "tmp3"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"tmp1", "=", 
      RowBox[{"Length", "[", 
       RowBox[{"Transpose", "[", "sequenz", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"tmp2", "[", "n", "]"}], "=", 
        RowBox[{"TableForm", "[", 
         RowBox[{
          RowBox[{"Transpose", "[", 
           RowBox[{"Take", "[", 
            RowBox[{
             RowBox[{"Transpose", "[", "sequenz", "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"n", ",", 
               RowBox[{"Min", "[", 
                RowBox[{"tmp1", ",", 
                 RowBox[{"n", "+", "rowLength", "-", "1"}]}], "]"}]}], 
              "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"TableSpacing", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"0.1", ",", "1"}], "}"}]}], ",", 
          RowBox[{"TableHeadings", "->", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
              "\"\<Daten: \>\"", ",", "\"\<W\[UDoubleDot]rfel: \>\"", ",", 
               "\"\<Viterbi: \>\""}], "}"}], ",", "None"}], "}"}]}]}], 
         "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"n", ",", "1", ",", "tmp1", ",", "rowLength"}], "}"}]}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"tmp3", "=", 
      RowBox[{"TableForm", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"tmp2", "[", "n", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "tmp1", ",", "rowLength"}], "}"}]}], 
        "]"}], "]"}]}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.574151551703125*^9, 3.574151571328125*^9}, {
  3.622880623473312*^9, 3.622880634324409*^9}, {3.622880677547229*^9, 
  3.622880719063373*^9}}],

Cell[TextData[{
 "Das erste Argument dieser Visualisierungsroutine ist eine dreiteilige Liste \
aus der Sequenz, der (bekannten) Abfolge interner Zust\[ADoubleDot]nde und \
der Variablen ",
 StyleBox["path",
  FontWeight->"Bold"],
 ", die die gerade ermittelte Viterbi-Decodierung enth\[ADoubleDot]lt. Dabei \
werden die Bezeichnungen durch die Elemente des HMM-Zustandsraums ersetzt."
}], "Text",
 CellChangeTimes->{{3.674625924951515*^9, 3.6746259574715624`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"res", "=", 
   RowBox[{"{", 
    RowBox[{"seq", ",", "label", ",", 
     RowBox[{"path", "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", "\[Rule]", "\"\<0\>\""}], ",", 
        RowBox[{"2", "\[Rule]", "\"\<F\>\""}], ",", 
        RowBox[{"3", "\[Rule]", "\"\<U\>\""}]}], "}"}]}]}], "}"}]}], 
  ";"}]], "Input"],

Cell[TextData[{
 "Das zweite Argument ist die Zeilenl\[ADoubleDot]nge, die zur Formatierung \
der Ausgabe verwendet wird. Neben der Formatierung besteht die Aufgabe dieser \
Routine nur darin, die entsprechende Zeilenbezeichnung \
hinzuzuf\[UDoubleDot]gen. Das Endergebnis dieses ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "-Exkurses lautet damit:"
}], "Text",
 CellChangeTimes->{{3.6746260084836373`*^9, 3.6746260189616528`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"makeViterbiOutput", "[", 
  RowBox[{"res", ",", "20"}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      TagBox[
       TagBox[GridBox[{
          {
           TagBox["\<\"Daten: \"\>",
            HoldForm], "6", "2", "3", "3", "6", "3", "2", "6", "6", "1", "1", 
           "5", "4", "5", "1", "2", "2", "6", "2", "6"},
          {
           TagBox["\<\"W\[UDoubleDot]rfel: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"},
          {
           TagBox["\<\"Viterbi: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxDividers->{
          "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{False}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.04]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        {OutputFormsDump`HeadedRows, None}],
       Function[BoxForm`e$, 
        TableForm[
        BoxForm`e$, TableSpacing -> {0.1, 1}, 
         TableHeadings -> {{"Daten: ", "W\[UDoubleDot]rfel: ", "Viterbi: "}, 
           None}]]]},
     {
      TagBox[
       TagBox[GridBox[{
          {
           TagBox["\<\"Daten: \"\>",
            HoldForm], "5", "5", "1", "1", "5", "5", "2", "3", "4", "4", "1", 
           "2", "1", "5", "3", "3", "2", "6", "5", "4"},
          {
           TagBox["\<\"W\[UDoubleDot]rfel: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"},
          {
           TagBox["\<\"Viterbi: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxDividers->{
          "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{False}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.04]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        {OutputFormsDump`HeadedRows, None}],
       Function[BoxForm`e$, 
        TableForm[
        BoxForm`e$, TableSpacing -> {0.1, 1}, 
         TableHeadings -> {{"Daten: ", "W\[UDoubleDot]rfel: ", "Viterbi: "}, 
           None}]]]},
     {
      TagBox[
       TagBox[GridBox[{
          {
           TagBox["\<\"Daten: \"\>",
            HoldForm], "1", "6", "6", "3", "3", "2", "3", "6", "3", "3", "5", 
           "2", "6", "5", "6", "6", "6", "6", "6", "5"},
          {
           TagBox["\<\"W\[UDoubleDot]rfel: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", \
"\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>"},
          {
           TagBox["\<\"Viterbi: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", \
"\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxDividers->{
          "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{False}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.04]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        {OutputFormsDump`HeadedRows, None}],
       Function[BoxForm`e$, 
        TableForm[
        BoxForm`e$, TableSpacing -> {0.1, 1}, 
         TableHeadings -> {{"Daten: ", "W\[UDoubleDot]rfel: ", "Viterbi: "}, 
           None}]]]},
     {
      TagBox[
       TagBox[GridBox[{
          {
           TagBox["\<\"Daten: \"\>",
            HoldForm], "2", "1", "6", "6", "3", "5", "6", "2", "6", "6", "3", 
           "6", "5", "5", "6", "3", "5", "4", "6", "3"},
          {
           TagBox["\<\"W\[UDoubleDot]rfel: \"\>",
            
            HoldForm], "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", \
"\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"},
          {
           TagBox["\<\"Viterbi: \"\>",
            
            HoldForm], "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", \
"\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", "\<\"U\"\>", \
"\<\"U\"\>", "\<\"U\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxDividers->{
          "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{False}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.04]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        {OutputFormsDump`HeadedRows, None}],
       Function[BoxForm`e$, 
        TableForm[
        BoxForm`e$, TableSpacing -> {0.1, 1}, 
         TableHeadings -> {{"Daten: ", "W\[UDoubleDot]rfel: ", "Viterbi: "}, 
           None}]]]},
     {
      TagBox[
       TagBox[GridBox[{
          {
           TagBox["\<\"Daten: \"\>",
            HoldForm], "1", "4", "3", "3", "4", "3", "4", "3", "4", "6", "2", 
           "5", "3", "3", "3", "4", "2", "6", "5", "1"},
          {
           TagBox["\<\"W\[UDoubleDot]rfel: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"},
          {
           TagBox["\<\"Viterbi: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxDividers->{
          "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{False}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.04]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        {OutputFormsDump`HeadedRows, None}],
       Function[BoxForm`e$, 
        TableForm[
        BoxForm`e$, TableSpacing -> {0.1, 1}, 
         TableHeadings -> {{"Daten: ", "W\[UDoubleDot]rfel: ", "Viterbi: "}, 
           None}]]]},
     {
      TagBox[
       TagBox[GridBox[{
          {
           TagBox["\<\"Daten: \"\>",
            HoldForm], "4", "3", "1", "1", "2", "1", "5", "3", "3", "1", "5", 
           "1", "3", "1", "3", "6", "3", "2", "3", "1"},
          {
           TagBox["\<\"W\[UDoubleDot]rfel: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"},
          {
           TagBox["\<\"Viterbi: \"\>",
            
            HoldForm], "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", \
"\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>", "\<\"F\"\>"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxDividers->{
          "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{False}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.04]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        {OutputFormsDump`HeadedRows, None}],
       Function[BoxForm`e$, 
        TableForm[
        BoxForm`e$, TableSpacing -> {0.1, 1}, 
         TableHeadings -> {{"Daten: ", "W\[UDoubleDot]rfel: ", "Viterbi: "}, 
           None}]]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.677571229057887*^9}]
}, Open  ]],

Cell[TextData[{
 "Offensichtlich identifiziert der Viterbi-Algorithmus unsere \
zwischenzeitliche Verwendung des unfairen W\[UDoubleDot]rfels fast \
elementgenau.\n\nMit Version 10 bietet ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " \[Dash] neben anderen Machine-Learning-Techniken \[Dash] auch eine \
eingebaute Umgebung f\[UDoubleDot]r Hidden-Markov-Modelle. Das oben \
beschriebene Ergebnis l\[ADoubleDot]sst sich daher auch direkt mit den \
Befehlen ",
 StyleBox["HiddenMarkovProcess",
  FontWeight->"Bold"],
 " und ",
 StyleBox["FindHiddenMarkovStates",
  FontWeight->"Bold"],
 " erzielen. Dazu bringen wir unser Modell zuerst in das f\[UDoubleDot]r ",
 StyleBox["HiddenMarkovProcess",
  FontWeight->"Bold"],
 " notwendige Format. Der Befehl erfordert (neben einer \
Anfangswahrscheinlichkeit f\[UDoubleDot]r jeden HMM-Zustand) eine \
\[CapitalUDoubleDot]bergangsmatrix und eine Emissionsmatrix. Beide Matrizen k\
\[ODoubleDot]nnen wir leicht \[UDoubleDot]ber die Befehle ",
 StyleBox["transition",
  FontWeight->"Bold"],
 " und ",
 StyleBox["emission",
  FontWeight->"Bold"],
 " abfragen. Der Nullzustand ist in diesem Format nicht mehr erforderlich und \
wird daher mit ",
 StyleBox["Drop",
  FontWeight->"Bold"],
 " aus der Liste ",
 StyleBox["state",
  FontWeight->"Bold"],
 " entfernt."
}], "Text",
 CellChangeTimes->{{3.6746260349926777`*^9, 3.6746260480246973`*^9}, {
  3.6746261058177824`*^9, 3.6746261067897835`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"init", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"0.5", ",", "0.5"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.622969027155911*^9, 3.622969032037149*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tMat", " ", "=", 
  RowBox[{
   RowBox[{"Outer", "[", 
    RowBox[{"List", ",", 
     RowBox[{"Drop", "[", 
      RowBox[{"state", ",", "1"}], "]"}], ",", 
     RowBox[{"Drop", "[", 
      RowBox[{"state", ",", "1"}], "]"}]}], "]"}], " ", "/.", " ", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n1_String", ",", "n2_"}], "}"}], "\[RuleDelayed]", "  ", 
    RowBox[{"transition", "[", 
     RowBox[{"n1", ",", "n2"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.622968768049594*^9, 3.622968814768927*^9}, {
   3.62296886103559*^9, 3.622968909248596*^9}, 3.622968987428443*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.95`", ",", "0.05`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.1`", ",", "0.9`"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.677571229120287*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eMat", " ", "=", " ", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{"Outer", "[", 
     RowBox[{"List", ",", "observation", ",", 
      RowBox[{"Drop", "[", 
       RowBox[{"state", ",", "1"}], "]"}]}], "]"}], " ", "/.", " ", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"n1_Integer", ",", " ", "n2_"}], "}"}], "\[RuleDelayed]", " ", 
     RowBox[{"emission", "[", 
      RowBox[{"n2", ",", "n1"}], "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.62296908737648*^9, 3.622969138020488*^9}, {
  3.6229695637846518`*^9, 3.6229696161343946`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.16666666666666666`", ",", "0.16666666666666666`", ",", 
     "0.16666666666666666`", ",", "0.16666666666666666`", ",", 
     "0.16666666666666666`", ",", "0.16666666666666666`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0.1`", ",", "0.1`", ",", "0.1`", ",", "0.1`", ",", "0.1`", ",", "0.5`"}],
     "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.677571229151487*^9}]
}, Open  ]],

Cell[TextData[{
 "Das Modell erh\[ADoubleDot]lt man dann einfach durch Anwenden von ",
 StyleBox["HiddenMarkovProcess",
  FontWeight->"Bold"],
 "."
}], "Text",
 CellChangeTimes->{{3.6746261693298764`*^9, 3.6746261780758886`*^9}, 
   3.6774923203280363`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hmm1", " ", "=", " ", 
  RowBox[{"HiddenMarkovProcess", "[", 
   RowBox[{"init", ",", " ", "tMat", ",", " ", "eMat"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.622964298828885*^9, 3.6229643014174957`*^9}, {
  3.622968732239382*^9, 3.622968735934331*^9}, {3.622969622077387*^9, 
  3.6229696318674173`*^9}}],

Cell[BoxData[
 RowBox[{"HiddenMarkovProcess", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.5`", ",", "0.5`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0.95`", ",", "0.05`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.1`", ",", "0.9`"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "0.16666666666666666`", ",", "0.16666666666666666`", ",", 
       "0.16666666666666666`", ",", "0.16666666666666666`", ",", 
       "0.16666666666666666`", ",", "0.16666666666666666`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "0.1`", ",", "0.1`", ",", "0.1`", ",", "0.1`", ",", "0.1`", ",", 
       "0.5`"}], "}"}]}], "}"}]}], "]"}]], "Output",
 CellChangeTimes->{3.67757123094549*^9}]
}, Open  ]],

Cell[TextData[{
 "Mit diesem Modell k\[ODoubleDot]nnen wir nun \[UDoubleDot]ber den Befehl ",
 StyleBox["FindHiddenMarkovStates",
  FontWeight->"Bold"],
 " die Sequenz ",
 StyleBox["seq",
  FontWeight->"Bold"],
 " decodieren:"
}], "Text",
 CellChangeTimes->{{3.6746261979629173`*^9, 3.6746262171959467`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"p1", " ", "=", " ", 
  RowBox[{"FindHiddenMarkovStates", "[", 
   RowBox[{"seq", ",", "hmm1", ",", "\"\<ViterbiDecoding\>\""}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6229697067753973`*^9, 3.6229697406295967`*^9}, {
  3.622970270210252*^9, 3.6229702720167027`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
   "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
   "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
   "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", 
   ",", "2", ",", "2", ",", "2", ",", "2", ",", "1", ",", "1", ",", "1", ",", 
   "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
   "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
   "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6775712314446907`*^9}]
}, Open  ]],

Cell["\<\
Das Ergebnis ist identisch zu dem Resultat aus unserer eigenen \
Implementierung.\
\>", "Text",
 CellChangeTimes->{{3.674626224394957*^9, 3.6746262344129715`*^9}}]
}, Open  ]]
},
WindowSize->{507, 900},
WindowMargins->{{636, Automatic}, {Automatic, 220}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> False}},
FrontEndVersion->"10.4 for Microsoft Windows (64-bit) (April 11, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 870, 36, 182, "Subsubtitle"],
Cell[CellGroupData[{
Cell[1453, 60, 270, 8, 94, "Subtitle"],
Cell[1726, 70, 755, 14, 220, "Text"],
Cell[2484, 86, 157, 5, 31, "Input"],
Cell[2644, 93, 425, 7, 68, "Text"],
Cell[3072, 102, 169, 5, 31, "Input"],
Cell[3244, 109, 508, 8, 106, "Text"],
Cell[3755, 119, 1320, 35, 212, "Input"],
Cell[5078, 156, 167, 3, 49, "Text"],
Cell[5248, 161, 515, 13, 92, "Input"],
Cell[5766, 176, 719, 17, 163, "Text"],
Cell[CellGroupData[{
Cell[6510, 197, 92, 2, 31, "Input"],
Cell[6605, 201, 91, 1, 31, "Output"]
}, Open  ]],
Cell[6711, 205, 253, 5, 49, "Text"],
Cell[CellGroupData[{
Cell[6989, 214, 102, 2, 31, "Input"],
Cell[7094, 218, 74, 1, 31, "Output"]
}, Open  ]],
Cell[7183, 222, 1024, 26, 163, "Text"],
Cell[8210, 250, 63, 1, 31, "Input"],
Cell[CellGroupData[{
Cell[8298, 255, 296, 8, 31, "Input"],
Cell[8597, 265, 624, 10, 72, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9258, 280, 436, 12, 72, "Input"],
Cell[9697, 294, 306, 6, 52, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10040, 305, 272, 8, 31, "Input"],
Cell[10315, 315, 624, 10, 72, "Output"]
}, Open  ]],
Cell[10954, 328, 265, 5, 68, "Text"],
Cell[11222, 335, 148, 4, 31, "Input"],
Cell[CellGroupData[{
Cell[11395, 343, 84, 2, 31, "Input"],
Cell[11482, 347, 72, 1, 31, "Output"]
}, Open  ]],
Cell[11569, 351, 470, 11, 106, "Text"],
Cell[12042, 364, 473, 14, 72, "Input"],
Cell[12518, 380, 125, 1, 30, "Text"],
Cell[12646, 383, 215, 6, 31, "Input"],
Cell[12864, 391, 206, 4, 49, "Text"],
Cell[13073, 397, 2212, 61, 272, "Input"],
Cell[15288, 460, 1676, 33, 353, "Text"],
Cell[CellGroupData[{
Cell[16989, 497, 380, 11, 72, "Input"],
Cell[17372, 510, 127, 3, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17536, 518, 363, 11, 72, "Input"],
Cell[17902, 531, 152, 3, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18091, 539, 380, 11, 72, "Input"],
Cell[18474, 552, 170, 4, 31, "Output"]
}, Open  ]],
Cell[18659, 559, 326, 7, 68, "Text"],
Cell[CellGroupData[{
Cell[19010, 570, 347, 10, 52, "Input"],
Cell[19360, 582, 180, 5, 33, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19577, 592, 256, 7, 52, "Input"],
Cell[19836, 601, 72, 1, 31, "Output"]
}, Open  ]],
Cell[19923, 605, 374, 8, 87, "Text"],
Cell[20300, 615, 111, 3, 31, "Input"],
Cell[20414, 620, 489, 14, 52, "Input"],
Cell[CellGroupData[{
Cell[20928, 638, 266, 7, 31, "Input"],
Cell[21197, 647, 1361, 20, 152, "Output"]
}, Open  ]],
Cell[22573, 670, 265, 5, 68, "Text"],
Cell[22841, 677, 2048, 54, 332, "Input"],
Cell[24892, 733, 465, 9, 125, "Text"],
Cell[25360, 744, 361, 11, 52, "Input"],
Cell[25724, 757, 440, 9, 106, "Text"],
Cell[CellGroupData[{
Cell[26189, 770, 96, 2, 31, "Input"],
Cell[26288, 774, 11430, 263, 254, "Output"]
}, Open  ]],
Cell[37733, 1040, 1441, 37, 334, "Text"],
Cell[39177, 1079, 199, 5, 31, "Input"],
Cell[CellGroupData[{
Cell[39401, 1088, 600, 15, 72, "Input"],
Cell[40004, 1105, 227, 7, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[40268, 1117, 580, 14, 92, "Input"],
Cell[40851, 1133, 452, 12, 72, "Output"]
}, Open  ]],
Cell[41318, 1148, 257, 7, 49, "Text"],
Cell[CellGroupData[{
Cell[41600, 1159, 329, 6, 31, "Input"],
Cell[41932, 1167, 776, 22, 112, "Output"]
}, Open  ]],
Cell[42723, 1192, 308, 9, 49, "Text"],
Cell[CellGroupData[{
Cell[43056, 1205, 295, 6, 52, "Input"],
Cell[43354, 1213, 1363, 20, 152, "Output"]
}, Open  ]],
Cell[44732, 1236, 173, 4, 49, "Text"]
}, Open  ]]
}
]
*)

